/**
 * @(#)workShop.java
 * @author 	Ken Flurchick <kenf@osc.edu>
 * Date:	03/10/2000
 * Purpose:	Maintains the attendeeList.xml file which
 *			keeps track of attendee information.
 *			Processes the workshops.xml file to generate
 *			workshop announcements.
 *			Stores and retrieves site information in the
 *			sites.xml file.
 * @version 1.00
 *
 * Required Classes:
 *		workshopParse
 */

import com.ibm.xml.*;
import com.ibm.xml.parser.*;
import com.ibm.xml.parsers.*;
import com.ibm.xml.parser.TXDocument;

import org.w3c.dom.*;
import org.w3c.dom.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.ProcessingInstruction;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Arrays.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;

import MyTableModel;

/**
 * Class: workShop
 * <p>
 * This program can perform several actions for
 * a workshop.  The methods maintain the three
 * XML files:
 * </p>
 * <ol>
 *	<li>attendeeList.xml file
 *	<li>workshops.xml file
 *	<li>sites.xml file
 * </ol>
 * <p>
 * In the workshopData directory of the workshop
 * user are the three XML files and two
 * subdirectories:
 * </p>
 * <ol>
 *	<li>tools/ - the JAVA tools for dealing with
 *		workshop information.
 *	<li>tmp/ - the directory to hold the temporary
 *		attendee registration information.
 * </ol>
 *
 */
public class workShop extends JFrame{
	JFrame parent;
/*
 * Miscellaneous class variables.
 */
	workshopParser pf = new workshopParser();

	String defaultPFilter = ".." + File.separator;
	String defaultPath = defaultPFilter + File.separator;
	String attendeeList = defaultPath + "workshopList.xml";
	String comments = defaultPath + "workshopComments.xml";
	String workshops = defaultPath + "Workshops.xml";
	String sites = defaultPath + "Sites.xml";
	String fileName = null;
	String lup, ts, oCts;
	String attendeeArray[][] = {{}};
	String tmp = "";
	String Name = "", school = "", email = "";
	int nWorkshops = 0;
	int nAttendees = 0;
	int wid = 0, nA = 0;
	Hashtable wList = new Hashtable();
	Hashtable aList = new Hashtable();
	JTree tree;
	Container c;
	DefaultTreeModel treeModel;
	DefaultMutableTreeNode root;
	JTable jt = new JTable();
	Vector headings = new Vector();
	Vector jj = new Vector();
	Vector tmpV = new Vector();

	// Strings and indices for workshop information.
	String oLoc = "", oKey = "", oDte = "", oDur = "", oOut = "";
	int sLI = 0, eLI = 0, sDaI = 0, eDaI = 0;
	int sKI = 0, eKI = 0, sDuI = 0, eDuI = 0;

	// Strings and indices for attendee information.
	int sFI = 0, eFI = 0, sMI = 0, eMI = 0;
	int slI = 0, elI = 0, siI = 0, eiI = 0;
	int sDI = 0, eDI = 0, sEI = 0, eEI = 0;

	// Set the headings for the attendee table.
	String title1;
/*
 * Panel variables.
 */
 	//Containers for attendee lists.
	JScrollPane scroll1 = new JScrollPane();
	JScrollPane scroll2 = new JScrollPane();
	JScrollPane scroll3 = new JScrollPane();
	JScrollPane scroll4 = new JScrollPane();
	JTextArea listWorkshops = new JTextArea();
	// JTextArea listAttendees = new JTextArea();
	JTable listAttendees = new JTable();
	JTextArea Comments = new JTextArea();
	JPanel workshopListPanel = new JPanel();
	JPanel workshopButtonPanel = new JPanel();
	JPanel attendeeListPanel = new JPanel();
	JPanel commentsPanel = new JPanel();

	JTabbedPane jtp = new JTabbedPane(SwingConstants.TOP);

	Font headerFontT = new Font("Serif", Font.BOLD,18);
	Font headerFontD = new Font("Serif", Font.PLAIN,16);
	Font highlightFont = new Font("Courier", Font.BOLD,14);
	Color highlightColor = new Color(255,0,0);
	Border border = BorderFactory.createBevelBorder(
		BevelBorder.RAISED);
/**
 * Constructor: workShop
 * <p>
 * Methods:
 * </p>
 * <ul type="square">
 *	<li>displayPanels()
 *	<li>workshopPanels()
 *	<li>sitePanel()
 *	<li>workshopDescPanel()
 * </ul>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
	public workShop () {
		super("Workshop Administrator Form");
/*
 * Set the headings vector names.
 */
		title1 = "Name";
		headings.addElement(title1);
		title1 = "School";
		headings.addElement(title1);
		title1 = "Email";
		headings.addElement(title1);
/*
 * Set the frame and panel sizes
 */
		int fw = 1250;
		int fh = 750;
		setSize(fw, fh);
		Color mColor = new Color(204, 255, 255);

		parent = this;

		c = getContentPane();
		c.setBackground(Color.white);
/*
 * Fill the panels (invoke the specific methods).
 */
		displayPanels();
		workshopPanels();
		// sitePanel();
		// descriptionPanel();

		JSplitPane jsp1 = new JSplitPane(
			JSplitPane.VERTICAL_SPLIT, workshopButtonPanel,
			workshopListPanel);

		JSplitPane jspA  = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, jsp1, attendeeListPanel);
/*
		JSplitPane jsp2 = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, jp21, jp22);

		JSplitPane jspB  = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, jsp2, usersPanel2);

		JSplitPane jsp3 = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, jp31, jp32);

		JSplitPane jspC  = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, jspA, usersPanel3);
*/

		jtp.addTab("Workshop Lists", jspA);
		// jtp.addTab("WorkShop Descriptions", jspB);
		// jtp.addTab("Workshops Sites", jspC);

		jtp.setForeground(new Color(204,104,104));
		jtp.setBackground(new Color(198,198,198));

		c.add(jtp);
	}
/**
 * Constructor: displayPanels
 * <p>
 * Generate the text panels to display information
 * for both workshops and attendees.
 * </p>
 * <ol>
 *	<li>attendee list text area.
 *	<li>workshop list text area.
 *	<li>Comments text area.
 * </ol>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
 	public void displayPanels(){
/**
 * Create the workshop list textArea panel
 */
		workshopListPanel.setLayout(new GridLayout(2,1));
		workshopListPanel.setBackground(new Color(204, 204, 255));
		workshopListPanel.setBorder(border);
		scroll1.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scroll1.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scroll1.getViewport().add(listWorkshops);
		listWorkshops.append(" ");
		workshopListPanel.add(scroll1);
/**
 * Build the workshop scroll pane - scroll3 to hold the
 * list of workshops tree.  A single node (root) tree to
 * start, and add this to scroll3.
 */
		root = new DefaultMutableTreeNode("List of Workshops.");
		treeModel = new DefaultTreeModel(root);
		tree = new JTree(treeModel);
		tree.setRootVisible(true);
		scroll3.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scroll3.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scroll3.getViewport().add(tree);
		scroll3.setVisible(true);
		workshopListPanel.add(scroll3);
/**
 * Build the attendee list panel.
 */
		attendeeListPanel.setLayout(new GridLayout(2,1));
		attendeeListPanel.setBackground(new Color(204, 204, 255));
		attendeeListPanel.setBorder(border);
		scroll2.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scroll2.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		// scroll2.getViewport().add(listAttendees);
		attendeeListPanel.add(scroll2);
/**
 * Build the comments panel.
 */
		scroll4.setVerticalScrollBarPolicy(
			ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scroll4.setHorizontalScrollBarPolicy(
			ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scroll4.getViewport().add(Comments);
		attendeeListPanel.add(scroll4);
	}
/**
 * Constructor: workshopPanels
 * <p>
 * Generate all the panels for the attendees:
 * </p>
 * <ol>
 *	<li>attendee list text area.
 *	<li>workshop list text area.
 *	<li>comments list text area.
 *	<li>Button area.
 * </ol>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
	public void workshopPanels(){
/**
 * Workshop list button panel.
 */
		workshopButtonPanel.setLayout(new GridLayout(4,2));
		workshopButtonPanel.setBackground(new Color(204, 204, 255));
		workshopButtonPanel.setBorder(border);

		JButton wrkList = new JButton("List All Workshops");
		workshopButtonPanel.add(new JLabel("List the workshops."));
		workshopButtonPanel.add(wrkList);

		// Build the workshop list list.
		wrkList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae1) {
				getWorkshopList();
			}
		});

		// List of attendees button.
		JButton attList = new JButton("List All Attendees");
		workshopButtonPanel.add(new
			JLabel("List the attendees for a workshops."));
		workshopButtonPanel.add(attList);

		attList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae1) {
				getAttendeeList();
			}
		});

		JButton emailList = new JButton("email list.");
		workshopButtonPanel.add(new JLabel("Generate an email list."));
		workshopButtonPanel.add(emailList);

		emailList.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae1) {
				// generateEmailList();
			}
		});
	}
/**
 * Method: 	getWorkshopList()
 * <p>
 * Process the Hashtable to list workshop information.
 * </p>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
 	public void getWorkshopList(){

		nA = 0;
		nWorkshops = pf.parseWorkshopList(wList);
		System.out.println("getWorkshopList nWorkshops = "
				+ nWorkshops);
		listWorkshops.append("List of Workshops.");
		if(nWorkshops == 0){
			System.out.println("Error in generating hash table");
			listWorkshops.append("No workshops found.\n");
		} else {
			listWorkshops.append("Last update : "
				+ (String)wList.get("lastUpdate") + "\n");
			listWorkshops.append(
				"The number of workshops\t" + nWorkshops + "\n");
			scroll1.getViewport().add(listWorkshops);

			// Extract the worksop attributes.
			oLoc = (String)wList.get("Location");
			oKey = (String)wList.get("Key");
			oDte = (String)wList.get("Date");
			oDur = (String)wList.get("Duration");
			oCts = (String)wList.get("Comments");

			// Build up the TreeNodes. Use DefaultMutableTreeNode
			// because the DefaultTreeModel can use that to build
			// a complete tree.
			for (int k = 0; k < nWorkshops; k++){
				DefaultMutableTreeNode subroot1 = new
					DefaultMutableTreeNode("Workshop " + (k + 1));
				eLI = oLoc.indexOf("::", sLI);
				oOut = "Location is " + oLoc.substring(sLI, eLI);
				sLI = eLI + 2;
				DefaultMutableTreeNode leaf1 = new
					DefaultMutableTreeNode(oOut);
				eKI = oKey.indexOf("::", sKI);
				oOut = "Key = " + oKey.substring(sKI, eKI);
				sKI = eKI + 2;
				DefaultMutableTreeNode leaf2 = new
					DefaultMutableTreeNode(oOut);
				eDaI = oDte.indexOf("::", sDaI);
				oOut = "Date = " + oDte.substring(sDaI, eDaI);
				sDaI = eDaI + 2;
				DefaultMutableTreeNode leaf3 = new
					DefaultMutableTreeNode(oOut);
				eDuI = oDur.indexOf("::", sDuI);
				oOut = "Duration = " + oDur.substring(sDuI, eDuI);
				sDuI = eDuI + 2;
				DefaultMutableTreeNode leaf4 = new
					DefaultMutableTreeNode(oOut);

				// Build our tree model starting at the root node,
				// and then make a JTree out of that.
				treeModel.insertNodeInto(subroot1, root, k);
				treeModel.insertNodeInto(leaf4, subroot1, 0);
				treeModel.insertNodeInto(leaf3, subroot1, 0);
				treeModel.insertNodeInto(leaf2, subroot1, 0);
				treeModel.insertNodeInto(leaf1, subroot1, 0);
				workshopListPanel.add(scroll3);
			}
			tree.addTreeSelectionListener(new TreeSelectionListener(){
				public void valueChanged(TreeSelectionEvent tse){
					TreePath tp = tse.getNewLeadSelectionPath();
					// listAttendees.setText("");
					Comments.setText("");
					if(!(jj.isEmpty())) {jj.removeAllElements();}
					if((tp.getLastPathComponent().
							toString()).startsWith("Workshop")){
						ts = tp.getLastPathComponent().toString();
						int gg = ts.indexOf(" ");
						int wNumber = Integer.parseInt(ts.substring(gg+1));
						System.out.println("wNumber = " + wNumber);
						// listAttendees.append("Selected "
						//	+ tp.getLastPathComponent() + "\n");
						pf.parseAttendeeList(wNumber-1, aList);
						System.out.println("aList from parseAttendeeList = "
							+ aList.toString());
						oCts = (String)aList.get("Comments");
						oCts = oCts.substring(0, oCts.indexOf("::"));
						Comments.append(oCts);
						tmp = (String)aList.get("nAttendees");
						tmp = tmp.substring(0, tmp.indexOf("::"));
						nA = Integer.parseInt(tmp);
						System.out.println("nA = " + nA);
						sFI = 0;
						siI = 0;
						sMI = 0;
						sDI = 0;
						sEI = 0;
						slI = 0;
						for(int l = 0; l < nA; l++){
							tmp = (String)aList.get("FirstName");
							eFI = tmp.indexOf("::", sFI);
							Name = tmp.substring(sFI, eFI);
							sFI = eFI + 2;
							tmp = (String)aList.get("MiddleName");
							eMI = tmp.indexOf("::", sMI);
							Name = Name + "  " + tmp.substring(sMI, eMI);
							sMI = eMI + 2;
							tmp = (String)aList.get("lastName");
							elI = tmp.indexOf("::", slI);
							Name = Name + "  " + tmp.substring(slI, elI);
							slI = elI + 2;
							tmp = (String)aList.get("Institution");
							eiI = tmp.indexOf("::", siI);
							school = tmp.substring(siI, eiI);
							siI = eiI + 2;
							tmp = (String)aList.get("Department");
							eDI = tmp.indexOf("::", sDI);
							school = school + "  " + tmp.substring(sDI, eDI);
							sDI = eDI + 2;
							tmp = (String)aList.get("Email");
							eEI = tmp.indexOf("::", sEI);
							email = tmp.substring(sEI, eEI);
							sEI = eEI + 2;
							System.out.println("Name = " + Name);
							System.out.println("School = " + school);
							System.out.println("email = " + email);
							if(!(tmpV.isEmpty())) {tmpV.removeAllElements();}
							tmpV.addElement(Name);
							System.out.println("tmpV = " + tmpV.toString());
							tmpV.addElement(school);
							System.out.println("tmpV = " + tmpV.toString());
							tmpV.addElement(email);
							System.out.println("tmpV = " + tmpV.toString());
							System.out.println("before add size of jj = " + jj.size());
							System.out.println("before add jj = " + jj.toString());
							jj.add(l, tmpV);
							System.out.println("size of jj = " + jj.size());
							System.out.println("jj = " + jj.toString());
						}
						JTable jt1 = new JTable(jj, headings);
						scroll2.getViewport().add(jt1);
					}
				}
			});
		}
/**
 * Build and display the attendee list.
 */
 		String usrFirst = (String)wList.get("FirstName");
 		String usrMiddle = (String)wList.get("MiddleName");
 		String usrLast = (String)wList.get("LastName");
 		String comments = (String)wList.get("Comments");
 		/*
 		if(nAttendees > 1){
 			int UstartIndex = 0;
 			int FstartIndex = 0;
 			int MstartIndex = 0;
 			int LstartIndex = 0;
 			int endIndex = 0;
 			for(int l = 0; l < nUsers; l++){
 				endIndex = usrs.indexOf("::", UstartIndex);
 				String usr = usrs.substring(UstartIndex, endIndex);
 				UstartIndex = endIndex + 2;
 				endIndex = usrFirst.indexOf("::", FstartIndex);
 				String fName = usrFirst.substring(FstartIndex, endIndex);
 				FstartIndex  = endIndex + 2;
 				endIndex = usrMiddle.indexOf("::", MstartIndex);
 				String mName = usrMiddle.substring(MstartIndex, endIndex);
 				MstartIndex  = endIndex + 2;
 				endIndex = usrLast.indexOf("::", LstartIndex);
 				String lName = usrLast.substring(LstartIndex, endIndex);
 				LstartIndex = endIndex + 2;

 				listUsers1.append(l + "   " + usr + "\t" + fName + " "
 					+ mName + " " + lName + "\n");
 				listUsers2.append(usr + "\t" + fName + " "
 						+ mName + " " + lName + "\n");
 			}
 		} else {
 			listUsers1.append(
 				"The number of user entries\t\t" + nUsers + "\n");
 			listUsers1.append(usrs + "\t\t" + usrFirst + " "
 				+ usrMiddle + " " + usrLast + "\n");
 			listUsers2.append(
 				"The number of user entries\t\t" + nUsers + "\n");
 			listUsers2.append(usrs + "\t\t" + usrFirst + " "
 				+ usrMiddle + " " + usrLast + "\n");
		}
		*/
	}
/**
 * Method: 	getAttendeeList()
 * <p>
 * Process the Hashtable to list attendee information.
 * </p>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
 	public void getAttendeeList(){

		wid = 1;
		nAttendees = pf.parseAttendeeList(wid, aList);
		System.out.println("getAttendeeList nWorkshops = "
				+ nAttendees);
		// listAttendees.append("List of Attendees.");
		if(nAttendees == 0){
			System.out.println("Error in generating hash table");
			// listAttendees.append("No attendees found.\n");
		} else {
			/*
			listAttendees.append("Last update : "
				+ (String)aList.get("lastUpdate") + "\n");
			listAttendees.append(
				"The number of workshops\t" + nWorkshops + "\n");
			listAttendees.append("Location : "
				+ (String)wList.get("Location") + "\n");
			listAttendees.append("Date : "
				+ (String)wList.get("Date") + "\n");
			listAttendees.append("Type : "
				+ (String)wList.get("Key") + "\n");
			listAttendees.append("Duration : "
				+ (String)wList.get("Duration") + "  days. \n");
			listAttendees.append("\n\n\n");
			*/
		}
	}
/**
 * Method: 	sitePanel()
 * <p>
 *
 * </p>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
	public void sitePanel(){
		String userName = null;
		String TAG = null;
	}
/**
 * Method:	workshopDescPanel
 * <p>
 *
 * </p>
 * @param	None
 * @return	None
 * @throws	None
 *
 */
	public void workshopDescPanel(){
	}
/**
 * Method: 	processWindowEvent()
 * <p>
 * Override the WindowEvent so that on a
 * window close event, the process exits.
 * </p>
 * @param	WindowEvent e
 * @return	None
 * @throws	None
 *
 */
	protected void processWindowEvent(WindowEvent e){
		super.processWindowEvent(e);
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			System.exit(0);
		}
	} // End of processWindowEvent method.
/**
 * Method: 	main
 * <p>
 * Instantiate the workShop class.
 * </p>
 * @param	String args[]
 * @return	None
 * @throws	None
 *
 */
	public static void main(String args[]) {
		workShop wS = new workShop();
		wS.setVisible(true);
	} // End of main method.
} // End of registereduser class.
