/**
 * @(#)		workshopParser.java
 * @author 	Ken Flurchick <kenf@osc.edu>
 * Date:	03/13/2000
 * Purpose:	Parse the workshop XML files.
 * @version 1.00
 */

import com.ibm.xml.*;
import com.ibm.xml.parser.*;
import com.ibm.xml.parsers.*;
import java.io.*;
import java.util.*;
import java.text.*;
import org.w3c.dom.*;
import com.ibm.xml.framework.*;

/**
 * Class:	workshopParser
 * <p>
 * Methods
 * </p>
 * <ul type="square">
 *	<li>parseWorkshopList
 *	<li>parseAttendeeList
 * 	<li>removeWorkshop
 *	<li>addWorkshop
 * </ul>
 * @param	None
 * @return	None
 * @throws	None
 */
public class workshopParser {
	Vector nVec = new Vector(); // nVec is the key vector.
	Vector vVec = new Vector(); // vVec is the value vector.
	static String value = null;
	Hashtable tHash = new Hashtable();
	Hashtable hash = new Hashtable();
	Hashtable cList = new Hashtable();
	Vector userKey = new Vector();
	Vector userValue = new Vector();
	Vector aTags = new Vector();
	Vector wTags = new Vector();
	Vector cTags = new Vector();
	int nWorkshops = 0;
	int nAttendees = 0;
	String [] wKeys = {"Location", "Date", "Key", "Duration"};
	String lup = null;
	Element item, itemA, itemB, r;
	Node a, w, c;

	String sPath = ".." + File.separator;
	String aFile = sPath + "workshopList.xml";
	String wFile = sPath + "tmp" + File.separator
		+ "workshopInfo.txt";
/**
 * Constructor: workshopParser
 * @param	None
 * @return	None
 * @throws	None
 */
	public void workshopParser(){
		// Empty Constructor
	}
/**
 * Method: 	parseWorkshopList
 * <p>
 * Find the current list of workshops in the
 * workshopAttendeeList.xml.
 * </p>
 * @param	Hashtable wList
 * @return	integer
 * @throws	None
 *
 */
	public int parseWorkshopList(Hashtable wList){

		nWorkshops = 0;
		String key = null, value = null, hName = null;
		NodeList nl0;
		String loc = "", dur = "", dte = "", ky = "", nA = "", cs = "";
		String ln = "", fn = "", In = "", mn = "", dept = "", em = "";

        try {
            // Open XML file
            FileInputStream is = new FileInputStream(aFile);

            // Start parsing
            Parser parser = new Parser(aFile);

            // Check if there is errors
            if (parser.getNumberOfErrors() > 0) {
				System.out.println("Errors > zero");
                return(2);
            }
            // Document is well-formed
            Document doc = parser.readStream(is);
            r = doc.getDocumentElement();

			// Build the DOM tree.
			removeWhiteSpaceNodes((Element)r);
			nl0 = r.getChildNodes();
			buildNodes(nl0, hName);

            // Build the workshop list information.
			wTags = getAllTags(r, "Workshop");
			nWorkshops = wTags.size();
			wList.put("lastUpdate", r.getAttribute("lastUpdate"));
			for(int l = 0; l < nVec.size(); l++){
				wList.put(nVec.get(l), (vVec.get(l) + "::"));
			}
			// Make a Hashtable from the name/value
			// attributes for the Workshop tag.

			for (int j = 0; j < nWorkshops; j++) {
				w = (Node)wTags.elementAt(j);
				loc = loc + ((Element)w).getAttribute("Location") + "::";
				dur = dur + ((Element)w).getAttribute("Duration") + "::";
				dte = dte + ((Element)w).getAttribute("Date") + "::";
				ky = ky + ((Element)w).getAttribute("Key") + "::";

				aTags = getAllTags(w, "Attendee");
				nAttendees = aTags.size();
				nA = nA + Integer.toString(nAttendees) + "::";
				for(int k = 0; k < nAttendees; k++){
					a = (Node)aTags.elementAt(k);
					ln = ln + ((Element)a).getAttribute("LastName") + "::";
				}
			}
        } catch (Exception e) {
            e.printStackTrace();
        }
        wList.put("Location", loc);
        wList.put("Duration", dur);
        wList.put("Key", ky);
        wList.put("Date", dte);
		wList.put("nAttendees", nA);
		wList.put("lastName", ln);
		// System.out.println("wList Hashable = " + wList.toString());

		return nWorkshops;
	}
/**
 * Method: 	parseAttendeeList
 * <p>
 * Find the current list of attendees in the
 * workshopAttendeeList.xml.
 * </p>
 * @param	int WID, Hashtable aList
 * @return	integer
 * @throws	None
 *
 */
	public int parseAttendeeList(int WID, Hashtable aList){
		boolean status = false;
		nAttendees = 0;
		String key = null, value = null, hName = null;
		Hashtable wL = new Hashtable();
		String ln = "";

        try {
            // Open XML file
            FileInputStream is = new FileInputStream(aFile);

            // Start parsing
            Parser parser = new Parser(aFile);

            // Check if there is errors
            if (parser.getNumberOfErrors() > 0) {
				System.out.println("Errors > zero");
                return(2);
            }
            // Document is well-formed

            Document doc = parser.readStream(is);
            Element r = doc.getDocumentElement();

			wTags = getAllTags(r, "Workshop");
			nWorkshops = wTags.size();
			nVec.removeAllElements();
			vVec.removeAllElements();
			for (int j = 0; j < nWorkshops; j++) {
				if(j == WID){
					a = (Node)wTags.elementAt(j);
					aTags = getAllTags(a, "Attendee");
					nAttendees = aTags.size();
					for(int l = 0; l < nAttendees; l++){
						ln = ln +
							((Element)aTags.elementAt(l)).
							getAttribute("lastName") + "::";
					}
					aList.put("lastName", ln);
					aList.put("nAttendees",
						(Integer.toString(nAttendees) + "::"));
					// Build the DOM tree.
					removeWhiteSpaceNodes((Element)a);
					NodeList nl1 = a.getChildNodes();
					buildNodes(nl1, hName);
					for(int l = 0; l < nVec.size(); l++){
						aList.put(nVec.get(l), (vVec.get(l)
							+ "::"));
					}
					status = true;
				}
			}
        } catch (Exception e) {
            e.printStackTrace();
        }
		// System.out.println("aList Table = " + aList.toString());

		return nAttendees;
	}
/**
 * Method: 	removeWorkshop
 * <P>
 * Remove the specified user widF.
 * </P>
 * @param	int widF
 * @return	boolean status
 * @throws	None
 *
 */
    public boolean removeWorkshop(int widF) {
		String hName = null;
		boolean status = false;

		Document doc = new TXDocument();
		// Document doc = new Document();
        try {
			String f1 = "MM  dd  yyyy  hh  mm";
			String f2 = "MM/dd/yyyy  hh:mm";
			SimpleDateFormat in = new SimpleDateFormat(f1);
			SimpleDateFormat out = new SimpleDateFormat(f2);

			Calendar cal = Calendar.getInstance();
			String datestring = (
				cal.get(Calendar.MONTH) + 1) + "  "
					+ cal.get(Calendar.DAY_OF_MONTH) + "  "
					+ cal.get(Calendar.YEAR) + "  "
					+ cal.get(Calendar.HOUR) + "  "
					+ cal.get(Calendar.MINUTE);
			Date curr = in.parse(datestring);

            // Open XML file
            FileInputStream is = new FileInputStream(aFile);

            // Start parsing
            Parser parser = new Parser(aFile);

            // Check if there is errors
            if (parser.getNumberOfErrors() > 0) {
				System.out.println("Errors > zero");
				status = false;
                return status;
            }
            // Document is well-formed

            doc = parser.readStream(is);
            Element r = doc.getDocumentElement();
            r.setAttribute("lastUpdate", out.format(curr));

            //Build the DOM tree.
			removeWhiteSpaceNodes(r);
			NodeList nl0 = r.getChildNodes();
			buildNodes(nl0, hName);

			Vector aTags = getAllTags(r, "Workshop");
			int m = aTags.size();
			for (int j = 0; j < m; j++) {
				Node a = (Node)aTags.elementAt(j);
				if( (Integer.parseInt(
						((Element)a).getAttribute("WID"))) == widF) {
					r.removeChild(a);
					status = true;
				}
			}
		} catch (Exception e) {
			System.out.println(e+" while removing user" );
 			status = false;
		}
		// Save the XML document
		// System.out.println("aFile = " + aFile);
		try{
			PrintWriter Out = new PrintWriter(
				new FileWriter(aFile, false));
			((TXDocument)doc).printWithFormat(Out);
		} catch(IOException e){
			System.out.println("Error opening Out.");
			System.exit(0);
		}
		return status;
	}
/**
 * Method: 	addWorkshop
 * <p>
 * Add the new workshop plus attendees
 * and comments to the XML file.
 * </p>
 * @param	None
 * @return	boolean status
 * @throws	None
 */
    public boolean addWorkshop() {
		Hashtable aInfo = null;
		Hashtable wInfo = null;
		boolean status = false;
		Document doc = new TXDocument();
		String[] attendees1;
		String[] attendees;
		String hName = null;
		String wName, aName, name;

		File here = new File("../tmp");

		// System.out.println("sFile = " + sFile);
/**
 * Get the list of files in the dirctory
 * Remove the workshopInfo file (not an attendee).
 */
		attendees = here.list();
		int j = 0;
		for(int i = 0; i < attendees.length; i++){
			name = attendees[i];
			if (!(name.equals("workshopInfo.txt"))){
				attendees[j] = name;
				j++;
			}
		}

		try{
			String f1 = "MM  dd  yyyy  hh  mm";
			String f2 = "MM/dd/yyyy  hh:mm";
			SimpleDateFormat in = new SimpleDateFormat(f1);
			SimpleDateFormat out = new SimpleDateFormat(f2);

 			Calendar cal = Calendar.getInstance();
			String datestring = (
				  cal.get(Calendar.MONTH) + 1) + "  "
				+ cal.get(Calendar.DAY_OF_MONTH) + "  "
				+ cal.get(Calendar.YEAR) + "  "
				+ cal.get(Calendar.HOUR) + "  "
				+ cal.get(Calendar.MINUTE);
			Date curr = in.parse(datestring);
/**
 * Read the workshopInfoFile and store the hashtable.
 */
			try{
				BufferedReader workshopInfoFile = new BufferedReader(
					new FileReader(wFile));
				System.out.println("Reading from " + wFile);
				hName = workshopInfoFile.readLine();
				workshopInfoFile.close();
				int startIndex = hName.indexOf("=");
				int endIndex = hName.indexOf(",", startIndex+1);
				System.out.println("startIndex for location" + startIndex);
				wInfo.put(wKeys[0],
					hName.substring(startIndex+1,endIndex));
				startIndex = hName.indexOf("=", startIndex + 1);
				endIndex = hName.indexOf(",", startIndex+1);
				wInfo.put(wKeys[1],
					hName.substring(startIndex+1, endIndex));
				wInfo.put(wKeys[2],
					hName.substring(startIndex+1, endIndex));
				wInfo.put(wKeys[3],
					hName.substring(startIndex+1, endIndex));
				System.out.println("Read hashtable " + wInfo);
			} // End of try block for workshopInfoFile.
			catch(IOException e){
				System.out.println("Error opening workshopInfoFile.");
				System.exit(0);
			}
/**
 * Open the workshopList.xml file for updating.
 */
			try {

				// Open XML file
				FileInputStream is = new FileInputStream(aFile);

				// Start parsing
				Parser parser = new Parser(aFile);

				// Check if there is errors
				if (parser.getNumberOfErrors() > 0) {
					System.out.println("Errors > zero");
					status = false;
					return status;
				}
				// Document is well-formed

				doc = parser.readStream(is);
				Element r = doc.getDocumentElement();
				r.setAttribute("lastUpdate", out.format(curr));

				// Build the DOM tree.
				removeWhiteSpaceNodes(r);
				NodeList nl0 = r.getChildNodes();
				buildNodes(nl0, hName);

				//  Make <workshop> element and add it
				//  as a child of root
				item = doc.createElement("Workshop");
				item.setAttribute("Key",
					(String)wInfo.get("key"));
				item.setAttribute("Location",
					(String)wInfo.get("location"));
				item.setAttribute("Date",
					(String)wInfo.get("date"));
				item.setAttribute("Duration",
					(String)wInfo.get("duration"));
/**
 * Read the attendee hashtables.
 */
				for(int i = 0; i < attendees.length; i++){
					name = attendees[i];
					System.out.println("File number " + i
						+ "  has name  " + name);
				try{
					BufferedReader attendeeFile = new BufferedReader(
						new FileReader(name));
					System.out.println("Reading from " + name);
					hName = attendeeFile.readLine();
				    attendeeFile.close();

					// Make <attendee> element and add it
					// as a child of workshop
					itemA = doc.createElement("attendee");
					itemA.setAttribute("LastName",
						(String)aInfo.get("lastName"));

					// Make <firstName> element and add it
					// as a child of attendee
					itemB = doc.createElement("FirstName");
					itemB.appendChild(doc.createTextNode(
						(String)aInfo.get("firstName")));
					itemA.appendChild(itemB);

					// Make <middleName> element and add it
					// as a child of attendee
					itemB = doc.createElement("MiddleName");
					itemB.appendChild(doc.createTextNode(
						(String)aInfo.get("middleName")));
					itemA.appendChild(itemB);

					// Make <email> element and add it
					// as a child of attendee
					itemB = doc.createElement("Email");
					itemB.appendChild(doc.createTextNode(
						(String)aInfo.get("email")));
					itemA.appendChild(itemB);

					// Make <department> element and add it
					// as a child of attendee
					itemB = doc.createElement("Department");
					itemB.appendChild(doc.createTextNode(
						(String)aInfo.get("department")));
					itemA.appendChild(itemB);

					// Make <institution> element and add it
					// as a child of attendee
					itemB = doc.createElement("Institution");
					itemB.appendChild(doc.createTextNode(
						(String)aInfo.get("institution")));
					itemA.appendChild(itemB);

					//  Add <attendee> as a child of <workshop>
					item.appendChild(itemA);

					//  Add <workshop> as a child of <root>
					r.insertBefore((Node)item, null);

					} // End of try block for attendeeFile.
					catch(IOException e){
						System.out.println(e + "Error opening attendeeFile.");
						System.exit(0);
					}
				} // End of for loop.
			}
			catch(IOException e){
				System.out.println("Error opening wFileOut.");
				System.exit(0);
			}
		} catch (Exception e) {
			System.out.println(e+" while removing user" );
 			status = false;
		}

		// Save the XML document
		// System.out.println("Out = " + aFile);
		try{
			PrintWriter Out = new PrintWriter(
				new FileWriter(aFile, false));
			((TXDocument)doc).printWithFormat(Out);
		} catch(IOException e){
			System.out.println("Error opening Out.");
		}
		return status;
	}
/**
 * Method: 	removeWhiteSpaceNodes
 * <p>
 * Remove the white space associated with a node.
 * </p>
 * @param	Element parent
 * @return	None
 * @throws	None
 */
    public void removeWhiteSpaceNodes(Element parent) {
		Node nextNode = parent.getFirstChild();
        for (Node child = parent.getFirstChild();
				nextNode != null;) {
			child = nextNode;
			nextNode = child.getNextSibling();

            if (child instanceof TXText){
				if(((TXText)child).getIsIgnorableWhitespace()){
					parent.removeChild(child);
				}
            } else if(child instanceof TXElement){
				removeWhiteSpaceNodes((Element)child);
			}
		}
    }
/**
 * Method: 	buildNodes
 * <p>
 * Parse a node and return the name/value pairs
 * as two vectors:
 * </p>
 *  <ol>
 *	<li>nVec - vector of keys.
 *	<li>vVec - vector of values
 * </ol>
 * @param	NodeList nl, String hName
 * @return	None
 * @throws	None
 */
    public void buildNodes(NodeList nl, String hName){
		int inl = 0;
		int inl2 = 0;
		int l;
		boolean found = false;

		while(inl < (nl.getLength())){
			Node childNL = nl.item(inl);
			String chName = childNL.getNodeName();
			if(chName != "#text") {
				hName = chName;
			}
			String chValue = childNL.getNodeValue();
			if (childNL instanceof TXText) {
				chValue = childNL.getNodeValue();
			} else if (childNL.hasChildNodes()){
				NodeList nl2 = childNL.getChildNodes();
				buildNodes(nl2, hName);
			}
			inl++;
			if((hName != null) && (chValue != null)) {
				if(nVec.isEmpty() == true){
					nVec.addElement(hName);
					vVec.addElement(chValue);
					found = false;
				} else if(nVec.isEmpty() != true){
					for(l = 0; l < nVec.size(); l++){
						if (hName.equals(nVec.get(l))){
							found = true;
							value = (String)vVec.get(l);
							value = value + "::" + chValue;
							vVec.setElementAt(value, l);
							break;
						}
					}
					if(found == false) {
						nVec.addElement(hName);
						vVec.addElement(chValue);
						found = false;
					}
				}
			}
		}
	}
/**
 * Method: 	getAlltags
 * <p>
 * Gett a Vector of nodes that have a specific node name.
 * </p>
 * @param	Node node, String tag
 * @return	Vector result
 * @throws	None
 */
	private Vector getAllTags(Node node, String tag) {
		Vector result=new Vector();

		for(Node ch = node.getFirstChild(); ch!=null;
				ch = ch.getNextSibling()) {
			if(ch instanceof Element) {
				if(ch.getNodeName().equals(tag)) {
					result.addElement(ch);
				}
			}
		}
		return result;
	}
}
